package com.loafle.cms.rpc.user.model;

import com.loafle.cms.proto.user.UserOuterClass;
import com.loafle.cms.rpc.common.util.Time;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import java.util.Date;
import java.util.Objects;

@Entity
@Table(name = "users")
public class User {
  @Id
  @GeneratedValue(generator = "user_generator")
  @SequenceGenerator(name = "user_generator", sequenceName = "user_sequence", initialValue = 1000)
  private Long id;

  @Basic
  @Column(name = "name", nullable = false, length = 255)
  private String name;

  @Basic
  @Column(name = "username", nullable = false, length = 150)
  private String username;

  @Basic
  @Column(name = "email", nullable = false, length = 100)
  private String email;

  @Basic
  @Column(name = "password", nullable = false, length = 100)
  private String password;

  @Basic
  @Column(name = "block", nullable = false, columnDefinition = "bool default false")
  private Boolean block;

  @Basic
  @Column(name = "send_email", nullable = true, columnDefinition = "bool default false")
  private Boolean sendEmail;

  @Basic
  @Column(name = "register_date", nullable = false)
  private Date registerDate;

  @Basic
  @Column(name = "lastvisit_date", nullable = false)
  private Date lastvisitDate;

  @Basic
  @Column(name = "activation", nullable = false, length = 100)
  private String activation;

  @Basic
  @Column(name = "params", nullable = false)
  private String params;

  @Basic
  @Column(name = "last_reset_time", nullable = false)
  private Date lastResetTime;

  @Basic
  @Column(name = "reset_count", nullable = false)
  private Long resetCount;

  @Basic
  @Column(name = "otp_key", nullable = false, length = 1000)
  private String otpKey;

  @Basic
  @Column(name = "otep", nullable = false, length = 1000)
  private String otep;

  @Basic
  @Column(name = "require_reset", nullable = true, columnDefinition = "bool default false")
  private Boolean requireReset;

  public Long getId() {
    return id;
  }

  public void setId(Long id) {
    this.id = id;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public String getUsername() {
    return username;
  }

  public void setUsername(String username) {
    this.username = username;
  }

  public String getEmail() {
    return email;
  }

  public void setEmail(String email) {
    this.email = email;
  }

  public String getPassword() {
    return password;
  }

  public void setPassword(String password) {
    this.password = password;
  }

  public Boolean getBlock() {
    return block;
  }

  public void setBlock(Boolean block) {
    this.block = block;
  }

  public Boolean getSendEmail() {
    return sendEmail;
  }

  public void setSendEmail(Boolean sendEmail) {
    this.sendEmail = sendEmail;
  }

  public Date getRegisterDate() {
    return registerDate;
  }

  public void setRegisterDate(Date registerDate) {
    this.registerDate = registerDate;
  }

  public Date getLastvisitDate() {
    return lastvisitDate;
  }

  public void setLastvisitDate(Date lastvisitDate) {
    this.lastvisitDate = lastvisitDate;
  }

  public String getActivation() {
    return activation;
  }

  public void setActivation(String activation) {
    this.activation = activation;
  }

  public String getParams() {
    return params;
  }

  public void setParams(String params) {
    this.params = params;
  }

  public Date getLastResetTime() {
    return lastResetTime;
  }

  public void setLastResetTime(Date lastResetTime) {
    this.lastResetTime = lastResetTime;
  }

  public Long getResetCount() {
    return resetCount;
  }

  public void setResetCount(Long resetCount) {
    this.resetCount = resetCount;
  }

  public String getOtpKey() {
    return otpKey;
  }

  public void setOtpKey(String otpKey) {
    this.otpKey = otpKey;
  }

  public String getOtep() {
    return otep;
  }

  public void setOtep(String otep) {
    this.otep = otep;
  }

  public Boolean getRequireReset() {
    return requireReset;
  }

  public void setRequireReset(Boolean requireReset) {
    this.requireReset = requireReset;
  }

  @Override
  public boolean equals(Object o) {
    if (this == o)
      return true;
    if (o == null || getClass() != o.getClass())
      return false;
    User user = (User) o;
    return id == user.id && block == user.block && resetCount == user.resetCount && Objects.equals(name, user.name)
        && Objects.equals(username, user.username) && Objects.equals(email, user.email)
        && Objects.equals(password, user.password) && Objects.equals(sendEmail, user.sendEmail)
        && Objects.equals(registerDate, user.registerDate) && Objects.equals(lastvisitDate, user.lastvisitDate)
        && Objects.equals(activation, user.activation) && Objects.equals(params, user.params)
        && Objects.equals(lastResetTime, user.lastResetTime) && Objects.equals(otpKey, user.otpKey)
        && Objects.equals(otep, user.otep) && Objects.equals(requireReset, user.requireReset);
  }

  @Override
  public int hashCode() {
    return Objects.hash(id, name, username, email, password, block, sendEmail, registerDate, lastvisitDate, activation,
        params, lastResetTime, resetCount, otpKey, otep, requireReset);
  }

  public UserOuterClass.User toProto() {
    return UserOuterClass.User.newBuilder().setId(getId()).setName(getName()).setUsername(getUsername())
        .setEmail(getEmail()).setPassword(getPassword()).setBlock(getBlock()).setSendEmail(getSendEmail())
        .setRegisterDate(Time.toTimestamp(getRegisterDate())).setLastvisitDate(Time.toTimestamp(getLastvisitDate()))
        .setActivation(getActivation()).setParams(getParams()).setLastResetTime(Time.toTimestamp(getLastResetTime()))
        .setResetCount(getResetCount()).setOtpKey(getOtpKey()).setOtep(getOtep()).setRequireReset(getRequireReset())
        .build();
  }

  public static User fromProto(UserOuterClass.User proto) {
    if (null == proto) {
      return null;
    }

    User model = new User();
    model.setId(proto.getId());
    model.setName(proto.getName());
    model.setUsername(proto.getUsername());
    model.setEmail(proto.getEmail());
    model.setPassword(proto.getPassword());
    model.setBlock(proto.getBlock());
    model.setSendEmail(proto.getSendEmail());
    model.setRegisterDate(Time.toDate(proto.getRegisterDate()));
    model.setLastvisitDate(Time.toDate(proto.getLastvisitDate()));
    model.setActivation(proto.getActivation());
    model.setParams(proto.getParams());
    model.setLastResetTime(Time.toDate(proto.getLastResetTime()));
    model.setResetCount(proto.getResetCount());
    model.setOtpKey(proto.getOtpKey());
    model.setOtep(proto.getOtep());
    model.setRequireReset(proto.getRequireReset());

    return model;
  }

}
