package com.loafle.cms.rpc.user.service;

import com.loafle.cms.proto.user.UserServiceGrpc;
import com.loafle.cms.proto.user.UserServiceOuterClass;
import com.loafle.cms.rpc.user.model.User;
import com.loafle.cms.rpc.user.repository.UserRepository;

import org.lognet.springboot.grpc.GRpcService;
import org.springframework.beans.factory.annotation.Autowired;

import io.grpc.stub.StreamObserver;

@GRpcService
public class UserService extends UserServiceGrpc.UserServiceImplBase {

  @Autowired
  private UserRepository userRepository;

  public void createUser(UserServiceOuterClass.CreateUserRequest request,
      StreamObserver<UserServiceOuterClass.CreateUserResponse> responseObserver) {

    try {
      User user = User.fromProto(request.getUser());
      user = userRepository.save(user);
      responseObserver.onNext(UserServiceOuterClass.CreateUserResponse.newBuilder().setId(user.getId()).build());
    } catch (Exception e) {
      responseObserver.onError(e);
    } finally {
      responseObserver.onCompleted();
    }
  }

  public void readUserById(UserServiceOuterClass.ReadUserByIdRequest request,
      StreamObserver<UserServiceOuterClass.ReadUserResponse> responseObserver) {

    try {
      User user = userRepository.findById(request.getId()).orElse(null);
      responseObserver.onNext(
          UserServiceOuterClass.ReadUserResponse.newBuilder().setUser(null == user ? null : user.toProto()).build());
    } catch (Exception e) {
      responseObserver.onError(e);
    } finally {
      responseObserver.onCompleted();
    }
  }

  public void readUserByUsername(UserServiceOuterClass.ReadUserByUsernameRequest request,
      StreamObserver<UserServiceOuterClass.ReadUserResponse> responseObserver) {

    try {
      User user = userRepository.findByUsername(request.getUsername());
      responseObserver.onNext(
          UserServiceOuterClass.ReadUserResponse.newBuilder().setUser(null == user ? null : user.toProto()).build());
    } catch (Exception e) {
      responseObserver.onError(e);
    } finally {
      responseObserver.onCompleted();
    }
  }

  public void updateUser(UserServiceOuterClass.UpdateUserRequest request,
      StreamObserver<UserServiceOuterClass.UpdateUserResponse> responseObserver) {
  }

  public void deleteUser(UserServiceOuterClass.DeleteUserRequest request,
      StreamObserver<UserServiceOuterClass.DeleteUserResponse> responseObserver) {
    try {
      userRepository.deleteById(request.getId());

      responseObserver.onNext(UserServiceOuterClass.DeleteUserResponse.newBuilder().setDeleted(0).build());
    } catch (Exception e) {
      responseObserver.onError(e);
    } finally {
      responseObserver.onCompleted();
    }
  }

  public void readAllUsers(UserServiceOuterClass.ReadAllUsersRequest request,
      StreamObserver<UserServiceOuterClass.ReadAllUsersResponse> responseObserver) {
  }
}